package com.zanxiang.sdk.common.miPay;

import com.alibaba.fastjson.JSONObject;
import com.zanxiang.common.domain.MiPayConfig;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.sdk.common.constant.RedisKeyConstant;
import com.zanxiang.sdk.common.util.HttpUtil;
import com.zanxiang.sdk.common.util.RedisUtil;
import com.zanxiang.sdk.service.Impl.pay.WxpayServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Objects;

/**
 * 支付
 *
 * @author xufeng
 * @date 2022/7/14 17:39
 */
@Slf4j
@Component
public class MiPayClient {

    private static final Logger logger = LoggerFactory.getLogger(MiPayClient.class);
    @Autowired
    private RedisUtil<String> redisUtil;

    private static final String method = "POST";

    private static final String contentType = "application/octet-stream; charset=utf-8";


    /**
     * 获取AccessToken
     *
     * @param appId
     * @param appSecret
     * @return AccessToken
     */
    private String getAccessToken(String appId, String appSecret) {
        String redisKey = RedisKeyConstant.WEIXIN_ACCESS_TOKEN + appId;
        String cache = redisUtil.getCache(redisKey);
        if (StringUtils.isNotEmpty(cache)) {
            return cache;
        }
        HashMap<String, String> param = new HashMap<>();
        param.put("grant_type", UrlConstants.CLIENT_CREDENTIAL);
        param.put("appid", appId);
        param.put("secret", appSecret);
        String result = HttpUtil.postData(UrlConstants.TOKEN_URL, param);
        if (StringUtils.isEmpty(result)) {
            return "";
        }
        JSONObject data = JSONObject.parseObject(result);
        if (Objects.isNull(data)) {
            return "";
        }
        Object accessToken = data.get("access_token");
        if (Objects.isNull(accessToken)) {
            return "";
        }
        String token = accessToken.toString();
        redisUtil.setCache(redisKey, token, Long.valueOf(data.get("expires_in").toString()));
        return token;
    }

    /**
     * 接口请求
     *
     * @param url         请求路径
     * @param params      请求参数
     * @param appSecret   密钥
     * @param midasAppKey 米大师key
     * @param sessionKey  session
     * @return
     */
    public HashMap<String, String> api(String url, RequestParam params, String midasAppKey, String sessionKey, String appSecret) {
        try {
            if (StringUtils.isEmpty(params.getOpenid()) ||
                    StringUtils.isEmpty(params.getAppid()) ||
                    StringUtils.isEmpty(params.getOffer_id()) ||
                    params.getTs() <= 0 || StringUtils.isEmpty(params.getZone_id())
            ) {
                throw new CustomException(HttpStatusEnum.PARAM_IS_NULL.getMsg() + JSONObject.toJSONString(params));
            }
            String accessToken = getAccessToken(params.getAppid(), params.getAccess_token());
            if (StringUtils.isEmpty(accessToken)) {
                throw new CustomException(HttpStatusEnum.ACCESS_TOKEN_CREATE_ERROR);
            }
            //使用沙箱时，沙箱地址替换
            if (params.getIs_sand() == 1) {
                url = url.replace("/cgi-bin/midas/", "/cgi-bin/midas/sandbox/");
            }
            HashMap<String, String> paramsMap = JSONObject.parseObject(JSONObject.toJSONString(params), HashMap.class);
            paramsMap.remove("is_sand");
            String sig = SnsSigCheck.makeSig(method, url, paramsMap, params.getApp_key());
            params.setSig(sig);
            params.setAccess_token(accessToken);
            url += "?access_token=" + accessToken;
            String result = HttpUtil.postData(url, JSONObject.toJSONString(params), contentType);
            logger.info("米大师接口调用失败 url:{} params:{} result:{}", url, params, result);
            if (StringUtils.isEmpty(result)) {
                throw new CustomException(HttpStatusEnum.MDS_CONNECT_ERROR);
            }
            HashMap<String, String> data = JSONObject.parseObject(result, HashMap.class);
            if (Objects.isNull(data)) {
                throw new CustomException(HttpStatusEnum.MDS_CONNECT_ERROR);
            }
            if (!data.get("errcode").equals("0")) {
                String errmsg = data.get("errmsg");
                Integer errcode = Integer.valueOf(data.get("errcode"));
                throw new CustomException(errmsg, errcode);
            }
            return data;
        } catch (RuntimeException e) {
            logger.error("米大师接口调用失败 url:{} params:{} e:{}", url, params, e);
            throw new RuntimeException("米大师接口调用失败");
        }
    }
}
