package com.zanxiang.sdk.controller;

import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.PayDeviceEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.sdk.common.annotation.ValidLogin;
import com.zanxiang.sdk.common.miPay.RequestParam;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.params.MPayBalanceParam;
import com.zanxiang.sdk.domain.params.OrderCheckInfoParam;
import com.zanxiang.sdk.domain.params.PreOrderParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.OrderCheckInfoVO;
import com.zanxiang.sdk.domain.vo.PreOrderMIPayConfigVO;
import com.zanxiang.sdk.domain.vo.PreOrderVO;
import com.zanxiang.sdk.service.GameStrategyService;
import com.zanxiang.sdk.service.MiPayService;
import com.zanxiang.sdk.service.PlatformOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author xufeng
 * @date 2022/6/17 17:07
 */
@Api(tags = "支付公共入口")
@RestController
@RequestMapping(value = "/order")
public class OrderController {
    @Autowired
    private PlatformOrderService orderService;

    @Autowired
    private GameStrategyService gameStrategyService;

    @Autowired
    private MiPayService miPayService;

    @ApiOperation(value = "订单生成")
    @PostMapping(value = "/preOrder")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PreOrderVO.class)})
    public ResultMap preOrder(@Validated @RequestBody PreOrderParam order, @ValidLogin UserData user) {
        String userId = String.valueOf(user.getUserId());
        PlatformOrderBO orderBo = BeanUtils.copy(order, PlatformOrderBO.class);
        orderBo.setUserId(userId);
        orderBo.setPayDevice(user.getDeviceType().toString());
        Boolean paySwitch = gameStrategyService.paySwitch(user, orderBo);
        String orderId = orderService.create(orderBo);
        if (StringUtils.isEmpty(orderId)) {
            return ResultMap.error("生成订单失败");
        }
        PreOrderVO data = new PreOrderVO();
        data.setOrderId(orderId);
        data.setPaySwitch(paySwitch);
        //todo 此处切换与产品定义切换含义不一致 游戏是否切换支付方式临时为true
        paySwitch = true;
        if (paySwitch) {
            RequestParam commonParam = miPayService.getCommonParam(Long.valueOf(order.getGameId()), String.valueOf(user.getUserId()));
            //密钥不返回出去
            commonParam.setApp_key("");
            data.setPreOrderMIPayConfigVO(BeanUtils.copy(commonParam, PreOrderMIPayConfigVO.class));
        }
        orderBo.setIsSwitch(paySwitch);

        return ResultMap.ok(data);
    }


    @ApiOperation(value = "小程序获取跳转支付信息")
    @PostMapping(value = "/checkInfo")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PreOrderVO.class)})
    public ResultMap checkInfo(@RequestBody OrderCheckInfoParam param, @ValidLogin UserData user) {
        //不知道干嘛用的，临时放
        String path = "pages/pay/index";
        OrderCheckInfoVO result = new OrderCheckInfoVO();
        result.setPath(path);
        PlatformOrderDTO orderInfo = orderService.info(param.getOrderId(), param.getUserId());
        if (PayDeviceEnum.MP.getCode().equals(user.getDeviceType())) {
            RequestParam commonParam = miPayService.getCommonParam(orderInfo.getGameId(), param.getUserId());
            if (StringUtils.isEmpty(commonParam.getAppid())) {
                return ResultMap.error("获取小程序appId");
            }
            result.setMpId(commonParam.getAppid());
        }
        return ResultMap.ok(result);
    }


    @ApiOperation(value = "【米大师】查询余额")
    @PostMapping(value = "/mPay_balance")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PreOrderVO.class)})
    public ResultMap mPayBalance(@RequestBody MPayBalanceParam mPayBalanceParam, @ValidLogin UserData user) {
        Boolean result = miPayService.midasGetBalance(user.getUserId(), mPayBalanceParam.getOrderId());
        return ResultMap.ok(result);
    }

}
