package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.exception.ParamNullException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.GameUser;
import com.zanxiang.mybatis.entity.GameUserRole;
import com.zanxiang.mybatis.mapper.GameUserRoleMapper;
import com.zanxiang.sdk.domain.params.GameUserRoleUpdateParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.GameUserRoleService;
import com.zanxiang.sdk.service.GameUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author : xufeng
 * @time : 2022-07-06
 */
@Slf4j
@Service
public class GameUserRoleImpl extends ServiceImpl<GameUserRoleMapper, GameUserRole> implements GameUserRoleService {

    @Autowired
    private GameUserService gameUserService;

    @Override
    public Boolean updateUserGameRole(GameUserRoleUpdateParam param, UserData userData) {
        //查询gameUserId
        if (StringUtils.isEmpty(param.getGameId())) {
            throw new ParamNullException("GameId");
        }
        Long gameId = Long.valueOf(param.getGameId());
        GameUser gameUserData = gameUserService.getOne(new LambdaQueryWrapper<GameUser>().eq(GameUser::getGameId, gameId).eq(GameUser::getUserId, userData.getUserId()).last("limit 1"));
        if (Objects.isNull(gameUserData)) {
            throw new CustomException(HttpStatusEnum.GAME_USER_UN_CREATE);
        }
        GameUserRole gameUserRoleWhere = new GameUserRole();
        gameUserRoleWhere.setRoleId(param.getRoleId());
        gameUserRoleWhere.setGameUserId(gameUserData.getId());
        gameUserRoleWhere.setUserId(userData.getUserId());

        GameUserRole gameUserRoleData = super.getOne(new LambdaQueryWrapper<GameUserRole>().setEntity(gameUserRoleWhere).last("limit 1"));
        GameUserRole roleData = BeanUtils.copy(param, GameUserRole.class);
        roleData.setOs(userData.getDeviceSystem());
        
        if (Objects.isNull(gameUserRoleData)) {
            //新增
            roleData.setRoleId(param.getRoleId());
            roleData.setGameUserId(gameUserData.getId());
            roleData.setUserId(userData.getUserId());
            super.save(roleData);
        } else {
            //更新
            super.update(roleData, new UpdateWrapper<GameUserRole>().setEntity(gameUserRoleWhere));
        }
        return true;
    }
}
