package com.zanxiang.sdk.service.Impl;

import com.alipay.easysdk.kernel.util.Signer;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.enums.CpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.common.utils.DateUtils;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.ObjectUtil;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.mybatis.entity.*;
import com.zanxiang.sdk.common.util.HttpUtil;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.mapper.OrderCompleteCreateMapper;
import com.zanxiang.sdk.service.*;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 订单履约逻辑
 *
 * @author xufeng
 * @date 2022/6/27 11:17
 */
@Component
@Slf4j
@Service
@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
public class PerformOrderServiceImpl implements PerformOrderService {
    protected final Logger logger = LoggerFactory.getLogger(PlatformOrderServiceImpl.class);

    @Autowired
    private PlatformOrderService platformOrderService;

    @Autowired
    private LogPayCpService logPayCpService;

    @Autowired
    private OrderCompleteCreateMapper orderCompleteMapper;

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @Autowired
    private GameUserService gameUserService;

    @Autowired
    private UserService userService;

    @Autowired
    private OrderCompleteService orderCompleteService;

    @Autowired
    private GameService gameService;

    @Autowired
    private GamePayWayService gamePayWayService;

    @Autowired
    private PayMerchantService payMerchantService;

    public Boolean pushCp(PlatformOrderDTO orderInfo) {
        logger.info("订单:{} 充值成功推送cp方回调 ----------start---------", orderInfo.getId());
        if (orderInfo.getCpStatus() == CpStatusEnum.SUCCESS.getCode()) {
            return true;
        }
        if (orderInfo.getGameId() <= 0) {
            logger.error("订单:{} 充值成功推送cp方回调失败，订单中GameId不存在", orderInfo.getId());
            return false;
        }
        String orderId = orderInfo.getId();

        //获取游戏中的app_key与app_key信息
        Game gameInfo = gameService.getById(orderInfo.getGameId());
        if (Objects.isNull(gameInfo)) {
            logger.error("订单:{} 充值成功推送cp方回调失败，订单中GameId:{} 的游戏信息未找到", orderInfo.getId(), orderInfo.getGameId());
            return false;
        }
        String app_key = gameInfo.getAppKey();
        String cpPaybackUrl = gameInfo.getCpPaybackUrl();
        if (StringUtils.isEmpty(gameInfo.getAppKey()) || StringUtils.isEmpty(gameInfo.getCpPaybackUrl())) {
            logger.error("订单:{} 充值成功推送cp方回调失败，订单中GameId:{} 的游戏信息中，AppKey:{} 或 CpPaybackUrl:{} 为空", orderInfo.getId(), orderInfo.getGameId(), app_key, cpPaybackUrl);
            return false;
        }
        HashMap<String, String> map = new HashMap<>();
        map.put("app_id", String.valueOf(orderInfo.getGameId()));
        map.put("cp_order_id", orderInfo.getCpOrderId());
        map.put("mem_id", orderInfo.getMgUserId());
        map.put("order_status", String.valueOf(orderInfo.getStatus()));
        map.put("pay_time", String.valueOf(orderInfo.getPayTime()));
        map.put("product_id", orderInfo.getProductId());
        map.put("product_price", String.valueOf(orderInfo.getAmount()));
        map.put("ext", orderInfo.getExt());
        map.put("app_key", app_key);
        TreeMap<String, String> keySort = new TreeMap<>(map);
        //拼接字符串
        String param = Signer.getSignCheckContent(keySort);
        //拼接app_key
        param += "&app_key=" + app_key;
        //生成sign
        String sign = MD5Util.MD5Encode(param, Constants.UTF8);
        map.remove("app_key");
        map.put("sign", sign);
        //拼接sign请求
        param += "&sign=" + sign;
        try {
            Integer i = orderInfo.getCpNotifyCnt() + 1;
            String s = HttpUtil.postData(cpPaybackUrl, param);
            Boolean pushSuccess = false;
            if (s.trim().toLowerCase(Locale.ROOT).equals(ResEnum.SUCCESS.getMsg())) {
                pushSuccess = true;
                //更新订单状态
                Order order = new Order();
                order.setId(orderId);
                order.setCpNotifyCnt(i);
                order.setLastCpNotifyTime(LocalDateTime.now());
                order.setCpStatus(pushSuccess ? CpStatusEnum.SUCCESS.getCode() : CpStatusEnum.FAILLE.getCode());
                platformOrderService.updateById(order);
            }
            LogPayCp logPayCp = new LogPayCp();
            logPayCp.setOrderId(orderId);
            logPayCp.setCpOrderId(orderInfo.getCpOrderId());
            logPayCp.setStatus(orderInfo.getStatus());
            logPayCp.setCpStatus(pushSuccess ? CpStatusEnum.SUCCESS.getCode() : CpStatusEnum.FAILLE.getCode());
            logPayCp.setCpPaybackUrl(cpPaybackUrl);
            logPayCp.setParams(param);
            logPayCp.setExt(orderInfo.getExt());
            logPayCp.setNotifyCnt(i);
            logPayCpService.save(logPayCp);
            logger.info("订单:{} 充值成功推送cp方回调 ----------end---------", orderInfo.getId());
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 充值成功推送cp方回调 失败 e:{}", orderId, e);
        }
        return false;
    }

    @Override
    public Boolean createOrderComplete(PlatformOrderDTO orderInfo) {
        logger.info("订单:{} 创建订单宽表 ----------start---------", orderInfo.getId());
        try {
            OrderComplete orderComplete = orderCompleteMapper.info(orderInfo.getId());
            orderComplete.setOrderDate(orderInfo.getCreateTime());
            //todo 推广相关信息
            orderComplete.setAgentId("");
            orderComplete.setAgentAccount("");
            orderComplete.setAgentMedia("");
            orderComplete.setAgentAdvert("");

            //todo 广告通知状态
            orderComplete.setAdvertNotifyStatus(0);
            orderComplete.setAdvertNotifyType(0);

            OrderComplete complete = orderCompleteService.getOne(new LambdaQueryWrapper<OrderComplete>().eq(OrderComplete::getOrderId, orderInfo.getId()));
            if (Objects.isNull(complete)) {
                orderCompleteMapper.insert(orderComplete);
                logger.info("订单:{} 生成宽表数据成功", orderInfo.getId());
            } else {
                OrderComplete pa = new OrderComplete();
                pa.setOrderId(complete.getOrderId());
                orderCompleteService.update(pa, new UpdateWrapper<OrderComplete>().setEntity(orderComplete));
                logger.info("订单:{} 宽表数据更新成功", orderInfo.getId());
            }
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 生成宽表数据失败 e:{}", orderInfo.getId(), e);
        }
        return false;
    }

    @Override
    public Boolean checkIsFirstRecharge(PlatformOrderDTO orderInfo) {
        logger.info("订单:{} 检查更新用户是否为首充 ----------start---------", orderInfo.getId());
        try {
            Order one = platformOrderService.getOne(new QueryWrapper<Order>().lambda()
                    .eq(Order::getUserId, orderInfo.getUserId())
                    .eq(Order::getStatus, OrderStateEnum.SUCCESS.getCode())
                    .ne(Order::getId, orderInfo.getId()).last("limit 1"));
            if (Objects.isNull(one)) {
                Order order = new Order();
                order.setId(order.getId());
                order.setIsFirstRecharge(1);
                platformOrderService.updateById(order);
                logger.info("订单:{} 验证订单是否为首充成功", orderInfo.getId());
                return true;
            }
        } catch (Exception e) {
            logger.error("订单:{} 验证订单是否为首充失败 e:{}", orderInfo.getId(), e);
        }
        return true;
    }

    @Override
    public Boolean userRechargeTotal(PlatformOrderDTO orderInfo) {
        logger.info("订单:{} 更新用户充值统计 ----------start---------", orderInfo.getId());
        try {
            //订单取消情况 做减法
            float amount = orderInfo.getAmount().floatValue();
            int num = 1;
            if (orderInfo.getStatus().equals(OrderStateEnum.CANCEL.getCode())) {
                amount = -amount;
                num = -1;
            }
            //游戏用户角色统计更新
            if (StringUtils.isNotEmpty(orderInfo.getRoleId()) && !"0".equals(orderInfo.getRoleId())) {
                GameUserRole gameUserRole = new GameUserRole();
                gameUserRole.setRoleId(orderInfo.getRoleId());
                gameUserRoleService.update(gameUserRole, new UpdateWrapper<GameUserRole>().lambda()
                        .set(GameUserRole::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUserRole)
                );
            } else {
                logger.info("订单:{} 用户玩家角色RoleId为空，不进行gameUserRole充值统计汇总", orderInfo.getId());
            }
            //游戏用户统计更新
            if (StringUtils.isNotEmpty(orderInfo.getMgUserId()) && !"0".equals(orderInfo.getMgUserId())) {
                GameUser gameUser = new GameUser();
                gameUser.setId(Long.valueOf(orderInfo.getMgUserId()));
                gameUserService.update(gameUser, new UpdateWrapper<GameUser>().lambda()
                        .set(GameUser::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(gameUser)
                );
            } else {
                logger.info("订单:{} 用户玩家MgUserId为空，不进行gameUser充值统计汇总", orderInfo.getId());
            }
            //用户统计更新
            if (StringUtils.isNotEmpty(orderInfo.getUserId()) && !"0".equals(orderInfo.getUserId())) {
                User user = new User();
                user.setId(Long.valueOf(orderInfo.getUserId()));
                userService.update(user, new UpdateWrapper<User>().lambda()
                        .set(User::getLastRechargeTime, orderInfo.getPayTime())
                        .setSql("recharge_count=recharge_count+" + num)
                        .setSql("recharge_money=recharge_money+" + amount)
                        .setEntity(user)
                );
            } else {
                logger.info("订单:{} 用户UserId为空，不进行User充值统计汇总", orderInfo.getId());
            }
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 用户充值统计逻辑操作失败 e:{}", orderInfo.getId(), e);
        }
        return false;
    }

    @Override
    public Boolean payMerchantTotal(PlatformOrderDTO orderInfo) {
        logger.info("订单:{} 更新商户号统计 ----------start---------", orderInfo.getId());
        try {
            if (StringUtils.isEmpty(orderInfo.getGamePaywayId())) {
                logger.error("订单:{} 更新商户号统计时，该订单GamePaywayId为空", orderInfo.getId());
                return false;
            }
            GamePayWay gamePayWayInfo = gamePayWayService.getInfo(orderInfo.getGamePaywayId());
            if (Objects.isNull(gamePayWayInfo)) {
                logger.error("订单:{} 更新商户号统计时，获取GamePayway表id:{}信息为空", orderInfo.getId(), orderInfo.getGamePaywayId());
                return false;
            }
            if (gamePayWayInfo.getPayMerchantId() <= 0) {
                logger.error("订单:{} 更新商户号统计时，获取GamePayway表id:{}信息中PayMerchantId为空", orderInfo.getId(), orderInfo.getGamePaywayId());
                return false;
            }
            PayMerchant payMerchantInfo = payMerchantService.getById(gamePayWayInfo.getPayMerchantId());
            if (Objects.isNull(payMerchantInfo)) {
                logger.error("订单:{} 更新商户号统计时，获取PayMerchant表id:{}信息为空", orderInfo.getId(), gamePayWayInfo.getPayMerchantId());
                return false;
            }
            BigDecimal amount = orderInfo.getAmount();
            Date update_time = payMerchantInfo.getDayUpdateDate();
            Date date = new Date();
            String totalPayAmount, mouthPayAmount, weakerPayAmount, dayPayAmount;
            totalPayAmount = " total_pay_amount = total_pay_amount + " + amount;
            mouthPayAmount = " mouth_pay_amount = mouth_pay_amount + " + amount;
            weakerPayAmount = " weaker_pay_amount = weaker_pay_amount + " + amount;
            dayPayAmount = " day_pay_amount = day_pay_amount + " + amount;
            if (update_time != null) {
                if (!DateUtils.isThisMonth(payMerchantInfo.getMouthUpdateDate())) {
                    mouthPayAmount = " mouth_pay_amount = " + amount;
                }

                if (!DateUtils.isThisWeek(payMerchantInfo.getWeakerUpdateDate())) {
                    weakerPayAmount = " weaker_pay_amount = " + amount;
                }

                if (!DateUtils.isToday(payMerchantInfo.getDayUpdateDate())) {
                    dayPayAmount = " day_pay_amount = " + amount;
                }
            }
            payMerchantService.update(new UpdateWrapper<PayMerchant>().lambda()
                    .set(PayMerchant::getMouthUpdateDate, date)
                    .set(PayMerchant::getWeakerUpdateDate, date)
                    .set(PayMerchant::getDayUpdateDate, date)
                    .setSql(totalPayAmount)
                    .setSql(mouthPayAmount)
                    .setSql(weakerPayAmount)
                    .setSql(dayPayAmount)
                    .setEntity(payMerchantInfo)
            );
            logger.info("订单:{} 更新商户号统计 ----------end---------", orderInfo.getId());
            return true;
        } catch (Exception e) {
            logger.error("订单:{} 更新商户号统计失败 e:{}", orderInfo.getId(), e);
        }
        return false;
    }
}
