package com.zanxiang.sdk.service;

import com.zanxiang.common.domain.ResultVo;
import com.zanxiang.sdk.domain.params.*;
import com.zanxiang.sdk.domain.vo.UserLoginVO;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description :
 */
public interface RegisterLoginService {

    /**
     * QQ授权登录
     *
     * @param response : 返回体
     */
    void qqLoginAuth(HttpServletResponse response);

    /**
     * QQ授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return : 返回登录token
     */
    ResultVo<UserLoginVO> qqLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);

    /**
     * 微信授权登录
     *
     * @param response : 返回体
     */
    void vxLoginAuth(HttpServletResponse response);

    /**
     * 微信授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return : 返回登录token
     */
    ResultVo<UserLoginVO> vxLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);

    /**
     * 用户名密码注册
     *
     * @param registerPasswordParam : 用户名密码注册参数
     * @param request               : request
     * @param userData              : userData
     * @return : 返回注册结果
     */
    ResultVo<UserLoginVO> registerPassword(RegisterPasswordParam registerPasswordParam, HttpServletRequest request, UserData userData);

    /**
     * 手机号注册
     *
     * @param registerMobileParam : 手机注册参数
     * @param request             : HttpServletRequest
     * @param userData            : userData
     * @return : 返回注册结果
     */
    ResultVo<UserLoginVO> registerMobile(RegisterMobileParam registerMobileParam, HttpServletRequest request, UserData userData);

    /**
     * 用户名密码登录
     *
     * @param userLoginParam : 登录参数
     * @param request        : HttpServletRequest
     * @param userData       : userData
     * @return : 返回登录token
     */
    ResultVo<UserLoginVO> loginPassword(LoginPasswordParam userLoginParam, HttpServletRequest request, UserData userData);

    /**
     * 手机号登录
     *
     * @param loginMobileParam : 手机号登录参数
     * @param request          : HttpServletRequest
     * @param userData         : userData
     * @return : 返回登录信息
     */
    ResultVo<UserLoginVO> loginMobile(LoginMobileParam loginMobileParam, HttpServletRequest request, UserData userData);
}
